package ufrn.imd.service.impl;

import lombok.AllArgsConstructor;
import lombok.extern.java.Log;
import ufrn.imd.domain.Account;
import ufrn.imd.domain.Client;
import ufrn.imd.repository.Repository;
import ufrn.imd.service.Service;

import java.rmi.RemoteException;
import java.util.Optional;

@Log
@AllArgsConstructor
public class WithdrawalServiceImpl implements Service {

    private final Repository<Client> clientRepository;

    public void withdraw(Double value, Optional<Client> clientOptional) throws RemoteException, RuntimeException {
        Client user = clientOptional
                .orElseThrow(() -> new RuntimeException("Null user!"));
        Client client = clientRepository.find(user.getId())
                .orElseThrow(() -> new RuntimeException("Invalid user!"));

        Account account = client.getAccount();
        if(value > account.getBalance()) throw new RuntimeException("Insufficient balance");

        account.setBalance(account.getBalance() - value);
        log.info("Withdrawal concluded!");
        log.info(String.format("Account balance: R$ %.2f", account.getBalance()));
    }
}
